!====================================================================
SUBROUTINE cal_corrector
  USE common
  IMPLICIT NONE
  INTEGER:: i
!
!--- updating nodal variable
!
  DO i=1,n_node
!
    IF(dof_ns(i)/=0) THEN
      us_n(i)=us_n(i)+grvec(dof_ns(i))
    END IF
!
  END DO
!
END SUBROUTINE cal_corrector
!====================================================================
SUBROUTINE cal_dndr(n_node_e,dndr_i)
  USE common
  IMPLICIT NONE
  INTEGER:: n_node_e
  REAL(p):: r
  REAL(p):: dndr_i(1:n_node_e)
!
  dndr_i=0.0_p
!
!--- dndřvZ
  r=lcoord_i
  SELECT CASE(n_node_e)
  CASE(2)
    dndr_i(1)=-0.5_p
    dndr_i(2)= 0.5_p
  CASE(3)
    dndr_i(1)= 0.5_p*(2.0_p*r-1.0_p)
    dndr_i(2)= 0.5_p*(2.0_p*r+1.0_p)
    dndr_i(3)=-2.0_p*r
  CASE DEFAULT
    WRITE(2,'(A)') "Wrong number of n_node_e in sub.cal_dndr"
    STOP
  END SELECT
!
END SUBROUTINE cal_dndr
!====================================================================
SUBROUTINE cal_dndz(n_node_e,dndz_i)
  USE common
  IMPLICIT NONE
  INTEGER:: n_node_e
  REAL(p):: dndr_i(1:n_node_e), dndz_i(1:n_node_e)
!
  dndr_i=0.0_p
  dndz_i=0.0_p
!
!--- RrǍvZ
  CALL cal_jac(n_node_e)
!
!--- dndřvZ
  CALL cal_dndr(n_node_e,dndr_i)
!
!--- dndžvZ
  dndz_i(1:n_node_e)= dndr_i(1:n_node_e)/jac_i
!
END SUBROUTINE cal_dndz
!====================================================================
SUBROUTINE cal_jac(n_node_e)
  USE common
  IMPLICIT NONE
  INTEGER:: n_node_e
  REAL(p):: dndr_i(1:n_node_e), coord_e(1:n_node_e)
!
  jac_i=0.0_p
  dndr_i=0.0_p
  coord_e=0.0_p
!
!--- dndřvZ
  CALL cal_dndr(n_node_e,dndr_i)
!
!--- vf̐ߓ_W̐ݒ
  coord_e(1:n_node_e)=coord_n(node_e(ielem,1:n_node_e))
!
!--- RrǍvZ
  jac_i= &
  DOT_PRODUCT(dndr_i(1:n_node_e),coord_e(1:n_node_e))
!
END SUBROUTINE cal_jac
!====================================================================
SUBROUTINE cal_kinematic_var
  USE common
  IMPLICIT NONE
  REAL(p):: x_e(1:n_node_es), dndzs_i(1:n_node_es)
!
!--- SЂ݂̌vZ
  dndzs_i=0.0_p; x_e=0.0_p
!  CALL cal_jac(n_node_es)
  CALL cal_dndz(n_node_es,dndzs_i)
  x_e(1:n_node_es)=us_n(node_e(ielem,1:n_node_es))
  stn_i=DOT_PRODUCT(dndzs_i(1:n_node_es),x_e(1:n_node_es))
!
!--- OXebv̑Ђ݂̌vZ
  x_e=0.0_p
  x_e(1:n_node_es)= &
    us_n(node_e(ielem,1:n_node_es))-ust_n(node_e(ielem,1:n_node_es)) 
  del_stn_i=DOT_PRODUCT(dndzs_i(1:n_node_es),x_e(1:n_node_es))
!
END SUBROUTINE cal_kinematic_var
!====================================================================
SUBROUTINE cal_rvec_rs(anode)
  USE common
  IMPLICIT NONE
  INTEGER:: anode,i,dofn
  REAL(p):: rvec_i
!
  rvec_i=0.0_p
!
!--- ws_1
  rvec_i= rvec_i &
  + sts_i*dnadzs_i(anode)
!
!--- ws_2
  rvec_i= rvec_i &
  - dens_mix0_i*shapeas_i(anode)*body
!
!--- S̎cxNgւ̊i[
  dofn=dof_ns(node_e(ielem,anode))
  IF(dofn/=0) THEN
    CALL set_grvec(dofn,rvec_i)
  ELSEIF(dofn == 0) THEN
    nloadbb_n(node_e(ielem,anode))=nloadbb_n(node_e(ielem,anode)) &
      - rvec_i*jac_i*weight_i
  END IF
!
END SUBROUTINE cal_rvec_rs
!====================================================================
SUBROUTINE cal_shape(n_node_e,shape_i)
  USE common
  IMPLICIT NONE
  INTEGER:: n_node_e
  REAL(p):: r, shape_i(1:n_node_e)
!
  shape_i=0.0_p
!
!--- `֐̌vZ
  r=lcoord_i
  SELECT CASE(n_node_e)
  CASE(2)
    shape_i(1)=0.5_p*(1.0_p-r)
    shape_i(2)=0.5_p*(1.0_p+r)
  CASE(3)
    shape_i(1)=0.5_p*r*(r-1.0_p)
    shape_i(2)=0.5_p*r*(r+1.0_p)
    shape_i(3)=(1.0_p+r)*(1.0_p-r)
  CASE DEFAULT
    WRITE(2,'(A)') "Wrong number of n_node_e in sub.cal_shape"
    STOP    
  END SELECT
!
END SUBROUTINE cal_shape
!====================================================================
SUBROUTINE cal_sol_gauss
  USE common
  IMPLICIT NONE
  INTEGER:: i,j,k
  REAL(p):: fac,fx
!
!--- Oi
  DO i=1,n_dof
    IF(ABS(gtmat(i,i)) < zero) THEN
      WRITE(2,'(A,I10)') "??? gtmat(i,i)=0.0 i=", i
      STOP
    END IF
!
    DO j=i+1,n_dof
      fac=gtmat(j,i)/gtmat(i,i)
      grvec(j)=grvec(j)-fac*grvec(i)
      DO k=i+1,n_dof
        gtmat(j,k)=gtmat(j,k)-fac*gtmat(i,k)
      END DO
    END DO
  END DO
!
!--- ޑ
  DO i=n_dof,1,-1
    fx=grvec(i)
    IF(i/=n_dof) THEN
      DO j=i+1,n_dof
        IF(j>n_dof) EXIT
        fx=fx-gtmat(i,j)*grvec(j)
      END DO
    END IF
    grvec(i)=fx/gtmat(i,i)
  END DO
!
!--- updated nodal variable
!  WRITE(3,'(A,4E15.5)') "solut",rvec(1:n_vec)
!
!  rvec1(1:n_vec)=MATMUL(tmat1(1:n_vec,1:n_vec),rvec(1:n_vec))
!  WRITE(3,'(A,4E15.5)') "rvec1",rvec1(1:n_vec)
!
END SUBROUTINE cal_sol_gauss
!====================================================================
SUBROUTINE cal_state_var
  USE common
  IMPLICIT NONE
  REAL(p):: n_0s, dens_sr0
!
  n_0s=0.0_p; dens_sr0=0.0_p
!
!--- Ԍ̌vZ
  n_0s=prop_base_m(mat_e(ielem),1)
!  poro_i=n_0s+stn_i
  poro_i=(n_0s+stn_i)/(1.0_p+stn_i)
  voidr_i=poro_i/(1.0_p-poro_i)
!
!--- ̖̂x̌vZ
  dens_sr0=prop_base_m(mat_e(ielem),2)
  dens_mix_i=(1.0_p-poro_i)*dens_sr0
  dens_mix0_i=(1.0_p-n_0s)*dens_sr0
!
END SUBROUTINE cal_state_var
!====================================================================
SUBROUTINE cal_tmat_drsdus(anode,bnode)
  USE common
  IMPLICIT NONE
  INTEGER:: anode,bnode,i,j,k,dofn,dofm
  REAL(p):: tmat_i
!
  tmat_i=0.0_p
! 
!--- dwsda_1
  tmat_i=tmat_i &
  + dnadzs_i(anode)*dep_i*dnbdzs_i(bnode)
!
!--- S̐ڐW}gbNXւ̊i[
  dofn=dof_ns(node_e(ielem,anode))
  dofm=dof_ns(node_e(ielem,bnode))
  IF((dofn/=0).AND.(dofm/=0)) CALL set_gtmat(dofn,dofm,tmat_i)
!
END SUBROUTINE cal_tmat_drsdus
